exports.Surface = require('./script/surface');
exports.Path = require('./script/path');
exports.Shape = require('./script/shape');
exports.Group = require('./script/group');
exports.Text = require('./script/text');
exports.Font = require('./script/font');
require('./current').setCurrent(exports);
